// Copyright Take Vos 2022.
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at https://www.boost.org/LICENSE_1_0.txt)

/** @file unicode_script.hpp
 */

#pragma once

#include <cstdint>

namespace hi::inline v1 {

enum class unicode_script: uint8_t {
    Unknown,
    Common,
    Latin,
    Greek,
    Cyrillic,
    Armenian,
    Hebrew,
    Arabic,
    Syriac,
    Thaana,
    Devanagari,
    Bengali,
    Gurmukhi,
    Gujarati,
    Oriya,
    Tamil,
    Telugu,
    Kannada,
    Malayalam,
    Sinhala,
    Thai,
    Lao,
    Tibetan,
    Myanmar,
    Georgian,
    Hangul,
    Ethiopic,
    Cherokee,
    Canadian_Aboriginal,
    Ogham,
    Runic,
    Khmer,
    Mongolian,
    Hiragana,
    Katakana,
    Bopomofo,
    Han,
    Yi,
    Old_Italic,
    Gothic,
    Deseret,
    Inherited,
    Tagalog,
    Hanunoo,
    Buhid,
    Tagbanwa,
    Limbu,
    Tai_Le,
    Linear_B,
    Ugaritic,
    Shavian,
    Osmanya,
    Cypriot,
    Braille,
    Buginese,
    Coptic,
    New_Tai_Lue,
    Glagolitic,
    Tifinagh,
    Syloti_Nagri,
    Old_Persian,
    Kharoshthi,
    Balinese,
    Cuneiform,
    Phoenician,
    Phags_Pa,
    Nko,
    Sundanese,
    Lepcha,
    Ol_Chiki,
    Vai,
    Saurashtra,
    Kayah_Li,
    Rejang,
    Lycian,
    Carian,
    Lydian,
    Cham,
    Tai_Tham,
    Tai_Viet,
    Avestan,
    Egyptian_Hieroglyphs,
    Samaritan,
    Lisu,
    Bamum,
    Javanese,
    Meetei_Mayek,
    Imperial_Aramaic,
    Old_South_Arabian,
    Inscriptional_Parthian,
    Inscriptional_Pahlavi,
    Old_Turkic,
    Kaithi,
    Batak,
    Brahmi,
    Mandaic,
    Chakma,
    Meroitic_Cursive,
    Meroitic_Hieroglyphs,
    Miao,
    Sharada,
    Sora_Sompeng,
    Takri,
    Caucasian_Albanian,
    Bassa_Vah,
    Duployan,
    Elbasan,
    Grantha,
    Pahawh_Hmong,
    Khojki,
    Linear_A,
    Mahajani,
    Manichaean,
    Mende_Kikakui,
    Modi,
    Mro,
    Old_North_Arabian,
    Nabataean,
    Palmyrene,
    Pau_Cin_Hau,
    Old_Permic,
    Psalter_Pahlavi,
    Siddham,
    Khudawadi,
    Tirhuta,
    Warang_Citi,
    Ahom,
    Anatolian_Hieroglyphs,
    Hatran,
    Multani,
    Old_Hungarian,
    SignWriting,
    Adlam,
    Bhaiksuki,
    Marchen,
    Newa,
    Osage,
    Tangut,
    Masaram_Gondi,
    Nushu,
    Soyombo,
    Zanabazar_Square,
    Dogra,
    Gunjala_Gondi,
    Makasar,
    Medefaidrin,
    Hanifi_Rohingya,
    Sogdian,
    Old_Sogdian,
    Elymaic,
    Nandinagari,
    Nyiakeng_Puachue_Hmong,
    Wancho,
    Chorasmian,
    Dives_Akuru,
    Khitan_Small_Script,
    Yezidi,
    Cypro_Minoan,
    Old_Uyghur,
    Tangsa,
    Toto,
    Vithkuqi,
    // ISO 15924
    Afaka,
    Arabic_Nastaliq,
    Blissymbols,
    Cirth,
    Cyrillic_Old_Church_Slavonic,
    Egyptian_Demotic,
    Egyptian_Hieratic,
    Han_Bopomofo,
    Han_Simplified,
    Han_Traditional,
    Indus,
    Jamo,
    Japanese,
    Japanese_Syllabaries,
    Jurchen,
    Kawi,
    Khitan_Large_Script,
    Khutsuri,
    Korean,
    Kpelle,
    Latin_Fraktur,
    Latin_Gaelic,
    Leke,
    Loma,
    Mayan_Hieroglyphs,
    Moon,
    Nag_Mundari,
    Naxi_Dongba,
    Nakhi_Geba,
    Proto_Cuneiform,
    Proto_Elamite,
    Book_Pahlavi,
    Kligon,
    Proto_Sinaitic,
    Private_Use_aa,
    Private_Use_bx,
    Ranjana,
    Rongorongo,
    Sarati,
    Shuishu,
    Sunuwar,
    Symbols,
    Symbols_Emoji,
    Syriac_Estrangelo,
    Syriac_Western,
    Syriac_Eastern,
    Tengwar,
    Unwritten_Documents,
    Visible_Speech,
    Woleai,
    Mathematical_Notation,
    // Open-type
    Byzantine_Music,
    Music,
};

}
